#!/usr/bin/perl

## 
##  Copyright (C) Fujitsu Technology Solutions 2014
##  All rights reserved
##

# version string
our $version = '3.00.01';
# Version:	3.00.01
# Date:		2014-08-22(1)


use strict;
use warnings;
use Getopt::Long qw(:config no_ignore_case bundling);
use Pod::Usage;
use Net::SNMP;
#use Time::Local 'timelocal';
use Time::localtime 'ctime';
use utf8;

# USED other scripts
our $snmpScript = "tool_fujitsu_server.pl";
our $cimScript = "tool_fujitsu_server_CIM.pl";
our     $wbemcliScript = "check_fujitsu_server_CIM.pl";
our         $wsmanPerlBindingScript = "fujitsu_server_wsman.pl";


#-----------------------------------------------------------------------------------------
# NOT YET: 
#     [-M|--mode={sync,threaded}]
#=item [-M|--mode={sync,threaded}]
#
#Chose processing mode: synchron check of hosts or threaded check of hosts
#
# UNUSED:
#    [-e|--extended]
#=item -e|--extended
#
#Extended result information - This option is used in calls of the other scripts.
#Extended information are provider version.
#-----------------------------------------------------------------------------------------

=head1 NAME

discover_fujitsu_server.pl - Discovery and Generation of Configurations

=head1 SYNOPSIS

discover_fujitsu_server.pl 
  { -H|--host=<host> 
    [-U|--use=<mode>]
    { [-I|--inputfile=<filename> [--inputdir=<inputfiledir>]]
      | --ic|--inputcollection=<dir>
    }
    { [--onehost] | --ipv4-discovery | --hc|--hostcollection=<file>
    }
    { [--config] | --txt }
    [-O|--outputdir=<dir>]
    [--ctimeout=<connection timeout in seconds>]
    [-t|--timeout=<timeout in seconds>]
    [-v|--verbose=<verbose mode level>]
  } | [-h|--help] | [-V|--version] 

Discovery of hosts and generation of Nagios host configurations if wanted

=head1 OPTIONS

=over 4

=item -H|--host=<name-or-ip>

Host address as DNS name or ip address of the server 

=item [-U|--use=<mode>]

Select SNMP with "S", select any CIM-XML/WS-MAN variant with "CW",
select only WS-MAN usage with "W", select only CIM-XML with "C"

=item [-I|--inputfile=<filename> [--inputdir=<inputfiledir>]] | --ic|--inputcollection=<dir>

The input option file will be used for the tool_***.pl calls. In the files should
be options for these tools - e.g. credentials, port and transport type restrictions.
The discovery script tries to analyse if the content is meant for SNMP or CIM calls.

inputfiledir - directory path of the input option file

inputcollection - directory path in which a collection of multiple input option files

=item [--onehost] | --ipv4-discovery | --hc|--hostcollection=<file>

onehost:
Test only one host address. This is default.

ipv4-discovery:
Test for 256 servers for a given n.n.n. IPv4 address.
Test for a limited range servers for a given n.n.n.<firstn>-<lastn> IPv4 address.

hostcollection:
Test a list of host addresses written into a file.

=item [--config] | --txt

Generate simple text output or generate text output and Nagios host definition configuration.
Default is --config.

=item [--outputdir=<dir>]

Specify output directory for resulting text and config files. Default is "svout".

=item [--ctimeout=<connection timeout in seconds>]

Timeout for the connection test to the CIM service. Default is 30 seconds.
All values higher than 30 will be ignored.
ATTENTION: This is used for the call of the CIM script !

=item -t|--timeout=<timeout in seconds>

Timeout for the call of the other scripts for SNMP or CIM checks.
ATTENTION: This is no timeout for this script !

=item -v|--verbose=<verbose mode level>

Enable verbose mode (levels: 10).
Print additional information about inside script calls.

=item -V|--version

Print version information and help text.

=item -h|--help

Print help text.

=cut

# define states
#### TEXT LANGUAGE AWARENESS (Standard in Nagios-Plugin)
our @state = ('OK', 'WARNING', 'CRITICAL', 'UNKNOWN');

# init main options
our $argvCnt = $#ARGV + 1;
our $optHost = '';
our $optPort = undef;
our $optUseMode = undef;
our $optServiceMode = undef;	# E ESXi, L Linux, W Windows
our $optTransportType = undef;

our $optTimeout = 0;
our $optShowVersion = undef;
our $optHelp = undef;

# SNMP authentication
our $optUserName = undef; #SNMPv3 or CIM
our $optCommunity = undef; #SNMPv1, SNMPv2
our $optAuthKey = undef; #SNMPv3
our $optAuthPassword = undef; #SNMPv3
our $optAuthProt = undef; #SNMPv3
our $optPrivKey = undef; #SNMPv3 & CIM
our $optPrivPassword = undef; #SNMPv3
our $optPrivProt = undef; #SNMPv3

# CIM authentication
#our $optUserName = undef; 
our $optPassword = undef; 
our $optCert = undef; 
#our $optPrivKey = undef; 
our $optCacert = undef;

# global option
$main::verbose = 0;
$main::verboseTable = 0;
$main::processPrint = 1;

# option input files
our $optInputFile	= undef;
our $optInputDir	= undef;
our $optInputCollection = undef;

# action options
our $optOneHost		= undef;
our $optIpv4Discovery	= undef;
our $optHostCollection	= undef;

# output options
our $optExtended	= undef;
our $optNoProcessPrint	= undef;
our $optTxtOut		= undef;
our $optConfigOut	= undef;
our $optOutdir		= undef;
our $optConnectTimeout	= undef;

# process options
our $optProcessMode	= undef;

# init output data
our $msg = '';
our $longMessage = '';
our $exitCode = 3;
our $variableVerboseMessage = '';
our $notifyMessage = '';

# init some multi used processing variables
$main::scriptPath = undef;
our @gCntCodes = ( 0,0,0,0 );
	####our @gCodesText = ( "ok", "no-snmp", "no-snmp-access", "unknown");
our $usableSNMP = undef;
our $usableCIM = undef;
our $usableCHKCIM = undef;
our $usableWSMAN = undef;
our $refInputFiles = undef;	# reference on a hash table !
our $refResults = undef;	# reference on a hash table !

###############################################################################
# PRINT FUNCTIONS
  sub finalizeMessageContainer {
	if ($msg) {
	    $msg =~ s/^\s*//gm; # remove leading blanks
	    $msg =~ s/^\-\s*/- /gm; # remove leading blanks
	}
	if ($notifyMessage) {
	    $notifyMessage =~ s/^\s*//gm; # remove leading blanks
	    $notifyMessage =~ s/\s*$//m; # remove last blanks
	    $notifyMessage =~ s/\n$//m; # remove last break
	}
	$notifyMessage = undef if ($main::verbose < 1 and ($exitCode==0));
	if ($longMessage) {
	    $longMessage =~ s/^\s*//m; # remove leading blanks
	    $longMessage =~ s/\s*$//m; # remove last blanks
	    $longMessage =~ s/\n$//m; # remove last break
	}
	if ($variableVerboseMessage) {
	    $variableVerboseMessage =~ s/^\s*//m; # remove leading blanks
	    $variableVerboseMessage =~ s/\n$//m; # remove last break
	}
	$variableVerboseMessage = undef if ($main::verbose < 2 and !$main::verboseTable);
	$longMessage = undef if ($longMessage and $longMessage eq '');
  } #finalizeMessageContainer

  sub finalize {
	my $exitCode = shift;
	print "@_" if (defined @_);
	print "\n";
	#alarm(0); # stop timeout
	exit($exitCode);
  }

  sub addMessage {
	my $container = shift;
	my $string = "@_";
	if ($string) {
		$msg .= $string				if ($container =~ m/.*m.*/);
		$notifyMessage .= $string		if ($container =~ m/.*n.*/);
		$longMessage .= $string			if ($container =~ m/.*l.*/);
		$variableVerboseMessage .= $string	if ($container =~ m/.*v.*/);
	}
  } #addMessage
###############################################################################
# OPTIONS
  sub readDataFile {
	my( $fileName ) = shift;

	if (! $fileName) {
		$exitCode = 10;	
		print "readDataFile: no filename \n" if ($main::verbose >= 60);
		return undef;
	}

	print "readDataFile [$fileName] \n" if ($main::verbose >= 60);

	if (! -r $fileName) {
		$exitCode = 11;
		print "readDataFile: [$fileName] not found \n" if ($main::verbose >= 60);
		return undef;
	}
	
	my $fileText = undef;
	open (my $infile, "<", $fileName);
	if (!$infile) {
	    $exitCode = 12;
	    return undef;
	} else {
	    $fileText = join('', <$infile> );
	    close $infile;
	    print "readDataFile: fileText [$fileText]\n" if ($main::verbose >= 60);
	}

	return ( $fileText );
  } #readDataFile
  sub getScriptOpts {	# script specific

	my $stringInput = shift;
	my $inputType = shift;

	my %options = ();

	print "getScriptOpts: stringInput = $stringInput\n" if ($stringInput && $main::verbose >= 60);
	print "getScriptOpts: inputType = $inputType\n" if ($inputType && $main::verbose >= 60);

	if (! $stringInput) {
    		GetOptions(\%options, 
		       	"t|timeout=i",	
		       	"v|verbose=i",	
		       	"V|version",	
		       	"h|help",

			"H|host=s",	
		        "U|use=s",	       	
			
		       	"onehost",
			"ipv4-discovery",
			"hc|hostcollection=s",  
			
			"e|extended",
			"ctimeout=i",	
			"txt",
			"config",
			"O|outputdir=s",
			"M|mode=s",
			
	   		"I|inputfile=s", 
			"inputdir=s",
			"ic|inputcollection=s",
		) or pod2usage({
			-msg     => "\n" . 'Invalid argument!' . "\n",
			-verbose => 1,
			-exitval => 3
		});
	}
    	return ( %options );
  } #getScriptOpts
  
  sub readOptions {

	my %mainOptions;	# command line optiond

	#
	# command line options first
	#
	%mainOptions = getScriptOpts();

	#
	if ($main::verbose >= 60) {
	    print "\n+++mainOptions at the end\n" if ($main::verbose >= 60);
	    foreach my $key_m (sort keys %mainOptions) {
		    print " $key_m = $mainOptions{$key_m}\n" if ($main::verbose >= 60);
	    }
	    print "+++\n" if ($main::verbose >= 60);
	}

	return ( %mainOptions);
  } #readOptions
  sub evaluateOptions { # script specific
	my $wrongCombination = undef;

	# check command-line parameters
	pod2usage(
		-verbose => 2,
		-exitval => 0,
	) if ((defined $optHelp) || !$argvCnt);

	pod2usage(
		-msg		=> "\n$0" . ' - version: ' . $version . "\n",
		-verbose	=> 0,
		-exitval	=> 0,
	) if (defined $optShowVersion);

	# option checks
	pod2usage(
		-msg		=> "\n" . 'Missing host address !' . "\n",
		-verbose	=> 0,
		-exitval	=> 3
	) if (!$optHost and !$optHostCollection);

	# wrong combinations
	$wrongCombination = "--onehost --ipv4-discovery" 
	    if ($optOneHost and $optIpv4Discovery);
	$wrongCombination = "--onehost --hostcollection" 
	    if (!$wrongCombination and $optOneHost and $optHostCollection);
	$wrongCombination = "--host=s --hostcollection=s" 
	    if (!$wrongCombination and $optHost and $optHostCollection);
	$wrongCombination = "-I=s --inputcollection=s" 
	    if (!$wrongCombination and $optInputFile and $optInputCollection);
	pod2usage({
		-msg     => "\n" . "Invalid argument combination \"$wrongCombination\"!" . "\n",
		-verbose => 0,
		-exitval => 3
	}) if ($wrongCombination);

	# TODO --- here is a blocking of not yet done parts
	if ($optProcessMode and $optProcessMode !~ m/sync/i) {
	    print "UNKNOWN - Only synchron processing enabled currently\n";
	    exit(3);
	}
	
	#
	if ($main::verbose > 100) {
		$main::verboseTable = $main::verbose;
		$main::verbose = 0;
	}

	# Defaults
	$optOutdir = "svout" if (!$optOutdir);
	$optOneHost = 999 if (!$optOneHost and !$optIpv4Discovery and !$optHostCollection);
	if (!defined $optTxtOut and !defined $optConfigOut) {
	    $optTxtOut = 999;
	    $optConfigOut = 999;
	}
  } #evaluateOptions
  sub handleOptions { # script specific
	# read all options and return prioritized
	my %options = readOptions();

	#
	# assign to global variables
	# for options like 'x|xample' the hash key is always 'x'
	#
	my $key = undef;
	$key="V";	$optShowVersion		= $options{$key} if ($options{$key});
	$key="h";	$optHelp		= $options{$key} if ($options{$key});
	$key="t";	$optTimeout		= $options{$key} if ($options{$key});
	$key="v";	$main::verbose		= $options{$key} if ($options{$key});

	$key="H";	$optHost		= $options{$key} if ($options{$key});
	$key="U";	$optUseMode 		= $options{$key} if ($options{$key});

	$key="onehost"; $optOneHost		= $options{$key} if ($options{$key});
	$key="ipv4-discovery";$optIpv4Discovery	= $options{$key} if ($options{$key});
	$key="hc";	$optHostCollection	= $options{$key} if ($options{$key});

	$key="e|extended";$optExtended		= $options{$key} if ($options{$key});
	$key="ctimeout";$optConnectTimeout	= $options{$key} if ($options{$key});
	$key="txt";	$optTxtOut		= $options{$key} if ($options{$key});
	$key="config";  $optConfigOut		= $options{$key} if ($options{$key});
	$key="O";	$optOutdir		= $options{$key} if ($options{$key});
	$key="M";	$optProcessMode		= $options{$key} if ($options{$key});

	$key="I";	$optInputFile		= $options{$key} if ($options{$key});
	$key="inputdir";$optInputDir	 	= $options{$key} if ($options{$key});
	$key="ic";	$optInputCollection	= $options{$key} if ($options{$key});

	# evaluateOptions expects options set in global variables
  	evaluateOptions();
  } #handleOptions

###############################################################################
# DIRECTORIES

  sub checkDirectory {
        my $dir = shift;
	my $modesDir = 0;
	$modesDir++ if ( -r $dir );
	$modesDir++ if ( -w $dir );
	$modesDir++ if ( -x $dir );
	$modesDir++ if ( -d $dir );
	if ($main::verbose >= 60 and $main::processPrint) {
	    print ">>> Check directory $dir [";
	    print "r" if ( -r $dir );
	    print "w" if ( -w $dir );
	    print "x" if ( -x $dir );
	    print "d" if ( -d $dir );
	    print "] <<<\n";
	}
	return $modesDir;
  } #checkDirectory

  sub listDirectory {
	my $dir = shift;
	my @sortArray = ();
	my @arr = ();

	opendir(my $dh, $dir);
    	while (my $file = readdir $dh) {
		next if ($file and ($file eq "." or $file eq ".."));
		next if ( -d "$dir/$file" );
        	print "<> $dir/$file\n" if ($main::verbose >= 60);
		my $filenm = "$dir/$file";
		push (@arr, $filenm );
    	}
    	closedir $dh if ($dh);
	@sortArray = sort @arr if ($#arr >= 0);
      return @sortArray;
  } #listDirectory

  sub handleOutputDirectory {
	my $modesOutDir = checkDirectory($optOutdir);
	if (!$modesOutDir) {
	    if (! mkdir $optOutdir, 0700) {
	    addMessage("m", 
		"ERROR - Can't create output directory $optOutdir");
	    $exitCode = 2;
	    return;
	    }
	} elsif ($modesOutDir < 4) {
	    addMessage("m", 
		"ERROR - output directory $optOutdir exists but has not enough access rights");
	    $exitCode = 2;
	    return;
	}
  } #handleOutputDirectory

###############################################################################
# INFRASTRUCTURE
  sub checkTools {
	my $fileName = undef; 

	$fileName = $main::scriptPath . $snmpScript;
	if (! -x $fileName) {
	    $usableSNMP = 0;
	} else {
	    $usableSNMP = 1;
	    $snmpScript = $main::scriptPath . $snmpScript;
	}

	$fileName = $main::scriptPath . $cimScript;
	if (! -x $fileName) {
	    $usableCIM = 0;
	} else {
	    $usableCIM = 1;
	    $cimScript = $main::scriptPath . $cimScript;
	}

	$fileName = $main::scriptPath . $wbemcliScript;
	if (! -x $fileName) {
	    $usableCHKCIM = 0;
	} else {
	    $usableCHKCIM = 1;
	}
	
	$fileName = $main::scriptPath . $wsmanPerlBindingScript;
	if (! -x $fileName) {
		$usableWSMAN = 0;
	} else {
		$usableWSMAN = 1;
	}

	# evaluate
	if (!$usableCIM and !$usableSNMP) {
	    $exitCode = 2;
	    addMessage("m", "ERROR - Unable to find tools for the SNMP or CIM access\n");
	    return;
	}
	if ($optUseMode and $optUseMode =~ m/^S/ and !$usableSNMP) {
	    $exitCode = 2;
	    addMessage("m", "ERROR - Unable to find tools for the SNMP access\n");
	    return;
	}
	if ($optUseMode and $optUseMode =~ m/^CW/ and (!$usableCIM or !$usableCHKCIM)) 
	{
	    $exitCode = 2;
	    addMessage("m", "ERROR - Unable to find tools for the CIM access\n");
	    return;
	}
	if ($optUseMode and $optUseMode =~ m/^W/ 
		and (!$usableCIM or !$usableCHKCIM or !$usableWSMAN)) 
	{
	    $exitCode = 2;
	    addMessage("m", "ERROR - Unable to find tools for the WS-MAN access\n");
	    return;
	}
	if ($optUseMode and $optUseMode =~ m/^C/ 
		and (!$usableCIM or !$usableCHKCIM)) 
	{
	    $exitCode = 2;
	    addMessage("m", "ERROR - Unable to find tools for the CIM-XML access\n");
	    return;
	}
  } #checkTools

  sub handleInputOptionFiles { # recursive call !
	my $chkFileName = shift;
	my $forSNMP = undef;		# check if file is for SNMP call 
	my $forCIM = undef;		# check if file is for CIM call 
	if (!$chkFileName and $optInputFile) {
		$chkFileName = "";
		$chkFileName .= $optInputDir . "/" if ($optInputDir 
		    and $optInputFile and $optInputFile !~ m/^\//);
		$chkFileName .= $optInputFile;
		$optInputFile = $chkFileName;
	}
	$exitCode = 3;
	if ($chkFileName) {
		print ">>> read input option file $chkFileName\n" if ($main::processPrint);
		my $content = readDataFile($chkFileName);
		if ($exitCode == 11) {
		    print "<<< ERROR - file not found or readable\n" 
			if ($main::processPrint);
		    addMessage("m", "- input file not found or readable");
		}
		elsif ($exitCode == 12) {
		    print "<<< ERROR - unable to open file\n" 
			if ($main::processPrint);
		    addMessage("m", "- unable to open input file");
		}
		elsif (!$content) {
		    print "<<< ERROR - empty file\n" 
			if ($main::processPrint);
		    addMessage("m", "- empty input file");
		    $exitCode = 10;
		}
		$exitCode = 2 if ($exitCode >= 10);
		return if ($exitCode == 2);
		
		$content =~ s/\n//mg;
		# TODO --- GetOptions Check !!!
		$forSNMP = 1 if ($content =~ m/\-\-authpassword/);
		$forSNMP = 1 if ($content =~ m/\-\-privpassword/);
		$forSNMP = 1 if ($content =~ m/\-C/);
		if (!$forSNMP) {
		    $forCIM = 1 if ($content =~ m/\-u/);
		    $forCIM = 1 if ($content =~ m/\-p/);
		    $forCIM = 1 if ($content =~ m/\-\-cert/);
		    $forCIM = 1 if ($content =~ m/\-\-cacert/);
		}

		$refInputFiles->{$chkFileName} = "SNMP" if ($forSNMP);
		$refInputFiles->{$chkFileName} = "CIM"  if ($forCIM);
		$refInputFiles->{$chkFileName} = "ANY"  if (!$forCIM and !$forSNMP);
		
		if ($main::processPrint) {
		    print "<<< OK - usable for SNMP calls\n" 
			if ($forSNMP);
		    print "<<< OK - usable for CIM calls\n" 
			if ($forCIM);
		    print "<<< WARNING - unspecific content - file will be ignored\n" 
			if (!$forCIM and !$forSNMP);
		}
	} # input file
	elsif ($optInputCollection) {
	    my @arrInput = ();
	    my $modesDir = checkDirectory($optInputCollection);
	    if (!$modesDir or $modesDir < 4) {   
		addMessage("m", 
		    "ERROR - input collection directory $optInputCollection doesn't exist or has not enough access rights");
		$exitCode = 2;
		return;
	    } 
	    @arrInput = listDirectory($optInputCollection);
	    foreach my $infile (@arrInput) {
		handleInputOptionFiles($infile);
	    }
	    return;
	} # input collection
  } #handleInputOptionFiles

  sub handleHostCollectionFile {
	my $colFile = shift;
	my @hostline = ();
	print ">>> read input option file $colFile\n" if ($main::processPrint);
	my $content = readDataFile($colFile);
	if ($exitCode == 11) {
	    print "<<< ERROR - file not found or readable\n" 
		if ($main::processPrint);
	    addMessage("m", "- host file not found or readable");
	}
	elsif ($exitCode == 12) {
	    print "<<< ERROR - unable to open file\n" 
		if ($main::processPrint);
	    addMessage("m", "- unable to open host file");
	}
	elsif (!$content) {
	    print "<<< ERROR - empty file\n" 
		if ($main::processPrint);
	    addMessage("m", "- empty host file");
	    $exitCode = 10;
	}
	$exitCode = 2 if ($exitCode >= 10);
	return @hostline if ($exitCode == 2);

	while ($content and $content =~ m/([^\n]+)\n/) {
	    push (@hostline, $1);
	    $content =~ s/[^\n]+\n//;
	}
	print "<<< OK\n" if ($main::processPrint);
	return @hostline;
  } #handleHostCollectionFile

  sub buildScriptCmd {
	my $script = shift;
        my $host = shift;
	my $infile = shift;
	return undef if (!$host);

	my $params = $script;
	$params .= " -H $host";	
	$params .= " -t $optTimeout"	    if ($optTimeout);
	$params .= " -I $infile"	    if ($infile);
	$params .= " --typetest --nopp";
	
	return $params;
  } #buildScriptCmd

  sub callScript {
	my $cmd = shift;
	my $result = undef;
	print "CMD: $cmd\n" if ($main::verbose >= 10);
	open (my $pHandle, '-|', $cmd);
	$result = join ('', <$pHandle>) if ($pHandle);
	close $pHandle if ($pHandle);
	return $result;
  } #callScript

  sub writeTxtFile {
	my $host = shift;
	my $type = shift;
	my $result = shift;

	my $txt = undef;
	my $txtFileName = $optOutdir . "/$host" . "_$type.txt";
	open ($txt, ">", $txtFileName);
	print $txt $result if ($result and $txt);
	close $txt if ($txt);
  } #writeTxtFile

  sub createSNMPconfig {
	my $host = shift;
	my $type = shift;
	my $result = shift;

	# 
	my $name = undef;
	my $serverType = undef;
	my $components = undef;
	my $subBlades = undef;
	my $admURL = undef;
	my $parent = undef;
	my $infile = undef;
	my $isServer		= undef;
	my $isBlade		= undef; # MMB
	my $isPrimequest	= undef; # MMB
	my $hasUpdateMonitor	= undef;
	my $isWindows		= undef;
	my $raid		= undef;

	# check server type
	if ($result =~ m/Type\s*= ([^\n]+)/) {
	    $serverType = $1;
	}
	$raid = 1 if ($result =~ m/RAID\s*= ([^\n]+)/); # ... Only RAID ?
	if (!$serverType) {
	    print "... no ServerView SNMP Agent information\n" 
		if ($main::processPrint);
	    my $topic = "CONNECTION OK - NO SERVERVIEW";
	    if ($refResults and $refResults->{$topic}) {
		my $store = $refResults->{$topic};
		$refResults->{$topic} = "$store + $host|SNMP";
	    } else {
		$refResults->{$topic} = "$host|SNMP";
	    }
	    return;
	}
	print "... SNMP ServerView information OK\n";

	# server type ?
	{
	    $isPrimequest = 1 if ($serverType eq "Primequest");
	    $isBlade = 1 if (!$isPrimequest and $serverType eq "Primergy Blade");
	    $isServer = 1 if (!$isPrimequest and !$isBlade);
	}

	# other values
	if ($result =~ m/Name\s*= ([^\n]+)/) {
	    $name = $1;
	}
	if ($isServer and $result =~ m/Components\s*= ([^\n]+)/) {
	    $components = $1;
	}
	if ($isBlade and $result =~ m/Sub-Blades\s*= ([^\n]+)/) {
	    $subBlades = $1;
	}
	if ($result =~ m/AdminURL\s*= ([^\n]+)/) {
	    $admURL = $1;
	}
	if ($result =~ m/OptionFile\s*= ([^\n]+)/) {
	    $infile = $1;
	}
	if ($isServer and $result =~ m/Parent Address\s*= ([^\n]+)/) {
	    $parent = $1;
	}
	if ($isServer) {
	    $hasUpdateMonitor = 1 if ($result =~ m/UpdateAgent\s*= [^\n]*available.*/);
	    $isWindows = 1 if ($result =~ m/OS\s*= Windows Server/);
	}
	$isWindows = 1 if (!$isServer);
	
	# name for output config file
	$name = "na" if (!$name);
	my $suffix = "SV";
	$suffix = "SViRMC" if ($serverType =~ m/iRMC/);
	if ($refResults and $refResults->{$name}) {
	    my $store = $refResults->{$name};
	    $refResults->{$name} = "$store + $host|SNMP|$suffix";
	} else {
	    $refResults->{$name} = "$host|SNMP|$suffix";
	}

	##############
	return if (!$optConfigOut);
	#... for tests: $name = 'I/N\A$R*I&I%I"I\'III';
	$name =~ s/[^A-Z,a-z,.,-,0-9]//g;
	my $cfg = undef;
	my $cfgFileName = $optOutdir . "/$host" . "_$type" . "_$name.cfg";
	
	# write config
	open ($cfg, ">", $cfgFileName);
	    print $cfg "####parents\t$parent\n" if ($parent);
	    print $cfg "define host {\n";
		print $cfg "\thost_name\t$host" . "_$name" . "_SNMP\n";
		print $cfg "\tdisplay_name\t$name\n";
		print $cfg "\taddress\t\t$host\n";
		my $hostgroups = undef;
		$hostgroups = "primergy-servers" if ($isServer);
		$hostgroups .= ",primergy-update-monitor" if ($isServer and $hasUpdateMonitor);
		$hostgroups = "primergy-blade-servers" if ($isBlade);
		$hostgroups = "primequest-servers" if ($isPrimequest);
		print $cfg "\thostgroups\t$hostgroups\n";
		my $useTemplate = undef;
		$useTemplate = "windows-server" if ($isWindows);
		$useTemplate = "linux-server" if (!$isWindows);
		print $cfg "\tuse\t\t$useTemplate\n";
		print $cfg "\t_SV_OPTIONS\t-I $infile\n" if ($infile);
		print $cfg "\tnotes_url\t$admURL\n" if ($admURL);
		print $cfg "\tregister\t1\n";
	    print $cfg "}\n";
	close $cfg if ($cfg);
  } #createSNMPconfig

  sub createCIMconfig {
	my $host = shift;
	my $type = shift;
	my $result = shift;

	# 
	my $name = undef;
	my $prot = undef;
	my $port = undef;
	my $trans = undef;
	my $srvmode = undef;
	my $components = undef;
	my $admURL = undef;
	my $parent = undef;
	my $infile = undef;
	my $hasUpdateMonitor	= undef;
	my $isWindows		= undef;
	my $hasModel = 0;

	# check components 
	if ($result =~ m/Components\s*= ([^\n]+)/) {
	    $components = $1;
	}
	if ($result =~ m/ServiceType\s*= ([^\n]+)/) {
	    $srvmode = $1;
	}
	#ATTENTION: iRMC Agentless/Agentless has no component list
	if ($srvmode and $srvmode =~ m/iRMC/ and $result =~ m/Model\s*= /) { 
	    $hasModel = 1;
	}
	if (!$components and !$hasModel) {
	    print "... no ServerView CIM Agent information (checked with summary status values)\n" 
		if ($main::processPrint);
	    my $topic = "CONNECTION OK - NO SERVERVIEW";
	    if ($refResults and $refResults->{$topic}) {
		my $store = $refResults->{$topic};
		$refResults->{$topic} = "$store + $host|CIM";
	    } else {
		$refResults->{$topic} = "$host|CIM";
	    }
	    return;
	}
	print "... CIM ServerView information OK\n";
	
	# get other values
	if ($result =~ m/Name\s*= ([^\n]+)/) {
	    $name = $1;
	}
	if ($result =~ m/Protocol\s*= ([^\n]+)/) {
	    $prot = $1;
	    $prot = undef if ($prot and $prot =~ m/CIM\-XML/);
	}
	if ($result =~ m/Port\s*= ([^\n]+)/) {
	    $port = $1;
	    $port = undef if ($port and $port =~ m/default/);
	}
	if ($result =~ m/TransType\s*= ([^\n]+)/) {
	    $trans = $1;
	    $trans = undef if ($trans and $trans =~ m/default/);
	}
	if ($result =~ m/AdminURL\s*= ([^\n]+)/) {
	    $admURL = $1;
	}
	if ($result =~ m/OptionFile\s*= ([^\n]+)/) {
	    $infile = $1;
	}
	if ($result =~ m/Parent Address\s*= ([^\n]+)/) {
	    $parent = $1;
	}
	{
	    $hasUpdateMonitor = 1 if ($result =~ m/UpdateAgent\s*= [^\n]*available.*/);
	    $isWindows = 1 if ($result =~ m/OS\s*= Windows Server/);
	}

	# name for output config file
	$name = "na" if (!$name);
	my $suffix = "SV";
	$suffix .= "iRMC" if ($srvmode =~ /iRMC/);
	if ($refResults and $refResults->{$name}) {
	    my $store = $refResults->{$name};
	    $refResults->{$name} = "$store + $host|CIM|$suffix";
	} else {
	    $refResults->{$name} = "$host|CIM|$suffix";
	}	
	
	###########################
	return if (!$optConfigOut);
	#... for tests: $name = 'I/N\A$R*I&I%I"I\'III';
	$name =~ s/[^A-Z,a-z,.,-,0-9]//g;
	my $cfg = undef;
	my $cfgFileName = $optOutdir . "/$host" . "_$type" . "_$name.cfg";

	# write config
	open ($cfg, ">", $cfgFileName);
	    print $cfg "####parents\t$parent\n" if ($parent);
	    print $cfg "define host {\n";
		print $cfg "\thost_name\t$host" . "_$name" . "_CIM\n";
		print $cfg "\tdisplay_name\t$name\n" if ($name ne "N.A." and $name !~ m/^N\/A$/i);
		print $cfg "\taddress\t\t$host\n";
		my $hostgroups = undef;
		$hostgroups = "primergy-servers-CIM";
		print $cfg "\thostgroups\t$hostgroups\n";
		my $useTemplate = undef;
		$useTemplate = "windows-server" if ($isWindows);
		$useTemplate = "linux-server" if (!$isWindows);
		print $cfg "\tuse\t\t$useTemplate\n";
		my $servicetype = undef;
		if ($srvmode) {
		    $servicetype = "I" if ($srvmode =~ m/iRMC/);
		    $servicetype = "E" if ($srvmode =~ m/ESXi/);
		    $servicetype = "L" if ($srvmode =~ m/Linux/i);
		    $servicetype = "W" if ($srvmode =~ m/Windows/i);
		}
		my $inoptions = undef;
		$inoptions .= "-UW " if ($prot);
		$inoptions .= "-P$port " if ($port);
		$inoptions .= "-T$trans " if ($trans);
		$inoptions .= "-S$servicetype " if ($servicetype);
		$inoptions .= "-I $infile ";
		print $cfg "\t_SV_CIM_OPTIONS\t$inoptions\n" if ($infile);
		print $cfg "\tnotes_url\t$admURL\n" if ($admURL);
		print $cfg "\tregister\t1\n";
	    print $cfg "}\n";
	close $cfg if ($cfg);
  } #createCIMconfig

###############################################################################
# DATA PROCESSING
  sub oneHostInfileLoop { # calls oneHost
	my $host = shift;
	my $refOptInputFiles = shift;
	my $snmpExitCode = 3;
	my $cimExitCode = 3;
	return if (!$optInputCollection or !$refOptInputFiles);
	
	my %allInputFiles = %$refOptInputFiles;
	{
	    my $hasSnmpInfile = undef;
	    foreach my $cInfile (sort keys %allInputFiles) 
	    {
		next if (!$cInfile or !$allInputFiles{$cInfile});
		my $cForWhat = $allInputFiles{$cInfile};
		$hasSnmpInfile = 1 if ($cForWhat eq "SNMP");
		last if ($hasSnmpInfile);
	    }
	    $exitCode = 3;
	    if ((!$optUseMode or $optUseMode =~ m/^S/)) 
	    {
		print "... try Community public\n" if ($main::processPrint);
		oneHost($host, undef, 0); # Test -C public
		$snmpExitCode = $exitCode;
	    }
	    foreach my $cInfile (sort keys %allInputFiles) 
	    {
		$exitCode = 3;
		next if (!$cInfile or !$allInputFiles{$cInfile} 
		    or $allInputFiles{$cInfile} eq "ANY");
		next if (!$allInputFiles{$cInfile} eq "CIM");
		my $cForWhat = $allInputFiles{$cInfile};
		if ((!$optUseMode or $optUseMode =~ m/^S/) and $snmpExitCode == 3
		and ($hasSnmpInfile and $cForWhat eq "SNMP")) 
		{
		    print "... try $cInfile\n" if ($main::processPrint);
		    oneHost($host, $cInfile, 0);
		    $snmpExitCode = $exitCode if ($exitCode != 3);
		} 
	    } 
	    #(sort { $allInputFiles{$b} cmp $allInputFiles{$a} } keys %allInputFiles) 
	    foreach my $cInfile (sort keys %allInputFiles) 
	    {
		$exitCode = 3;
		next if (!$cInfile or !$allInputFiles{$cInfile} 
		    or $allInputFiles{$cInfile} eq "ANY");
		next if (!$cInfile or !$allInputFiles{$cInfile} 
		    or $allInputFiles{$cInfile} eq "SNMP");
		my $cForWhat = $allInputFiles{$cInfile};
=begin COMMENT
		if ((!$optUseMode or $optUseMode =~ m/^S/) and $snmpExitCode == 3
		and ($hasSnmpInfile and $cForWhat eq "SNMP")) 
		{
		    print "... try $cInfile\n" if ($main::processPrint);
		    oneHost($host, $cInfile, 0);
		    $snmpExitCode = $exitCode if ($exitCode != 3);
		} els
=end COMMENT
=cut
		if ((!$optUseMode or $optUseMode =~ m/^C/ or $optUseMode =~ m/^W/) 
		and $cForWhat eq "CIM" and ($cimExitCode == 3 or $cimExitCode == 1)) 
		{
		    #$optUseMode = "CW" if (!$optUseMode);
		    print "... try $cInfile \n";
		    oneHost($host, $cInfile, 0);
		    $cimExitCode = $exitCode if ($exitCode != 3);
		    last if ($exitCode == 0 or $exitCode == 3);
		}
	    }
	    $exitCode = $snmpExitCode if ($snmpExitCode != 3);
	    $exitCode = $cimExitCode if 
		(defined $cimExitCode 
		and ($cimExitCode==0 or $cimExitCode==1) 
		and  $cimExitCode==0 < $exitCode);
	} 
  } #oneHostInfileLoop
  sub oneHost { # recursive call via oneHostInfileLoop !
	my $host = shift;
	my $infile = shift;
	my $start = shift;
	my $pp = $main::processPrint;
	my %allInputFiles = ();
	my $forWhat = undef;
	my $rc = 3;
	%allInputFiles = %$refInputFiles if ($refInputFiles);
	$forWhat = $allInputFiles{$infile} if ($refInputFiles and $infile and $allInputFiles{$infile});

	print ">>> $host\n" if ($main::processPrint and $start);
	if (!$infile and $optInputCollection and $start) {
	    oneHostInfileLoop($host, $refInputFiles); # recursive !
	} else { # single input file
	    my $log = undef;
	    my $logFileName = undef;
	    my $fileHost = $host;
	    #$host =~ s/[^A-Z,a-z,.,-,0-9]//g;
	    $fileHost =~ s/[^A-Z,a-z,.,\-,0-9]//g;
	    $logFileName = $optOutdir . "/$fileHost.log";
	    open ($log, ">>", $logFileName);

	    $pp = undef if (!$log);
	    
	    print $log "START DATE: \t" . ctime() . "\n";
	    print $log "ADDRESS:\t$host\n" if ($pp);
	    print $log "INFILE:\t\t$infile\n" if ($infile);
	    #### SNMP
	    if ((!$optUseMode or $optUseMode =~ m/^S/) 
	    and ((!$infile) or ($forWhat and $forWhat eq "SNMP"))) 
	    {
		
		print $log ">>> SNMP\n" if ($pp);
		my $snmpInfile = undef;
		$snmpInfile = $infile if ($infile and $forWhat and $forWhat eq "SNMP");
		my $cmd = buildScriptCmd($snmpScript, $host, $snmpInfile);
		$cmd .= " -e"; # to get parent address
		print $log "... call: $cmd \n" if ($pp);
		my $result = callScript($cmd);
		print $log $result if ($result and $pp);

		my $extract = 'unknown';
		$result =~ m/^([^\s]+) /;
		$extract = $1 if ($1);
		$rc = 0 if ($extract eq "OK");
		if ($rc == 0) {
		    $exitCode = 0;
		    print "... SNMP connection OK \n";
		    writeTxtFile($fileHost, "SNMP", $result);
		    createSNMPconfig($fileHost, "SNMP", $result);
		} # success
		elsif ($result =~ m/TIMEOUT/i) {
		    $extract = "TIMEOUT";
		    if ($refResults and $refResults->{"TIMEOUT"}) {
			my $store = $refResults->{"TIMEOUT"};
			$refResults->{"TIMEOUT"} = "$store + $host|SNMP";
		    } else {
			$refResults->{"TIMEOUT"} = "$host|SNMP";
		    }
		}
		print $log "<<< $extract\n" if ($pp);
	    }
	    #### CIM
	    $rc = 3;
	    if ((!$optUseMode or $optUseMode =~ m/^C/ or $optUseMode =~ m/^W/) and $infile
		and $forWhat and $forWhat eq "CIM") 
	    {
		print $log ">>> CIM\n" if ($pp);
		my $cimInfile = undef;
		$cimInfile = $infile;
		my $cmd = buildScriptCmd($cimScript, $host, $cimInfile);
		$cmd .= " -e"; # ... read agents version
		$cmd .= " -U$optUseMode" if ($optUseMode 
		    and ($optUseMode eq "C" or $optUseMode =~ m/^W/));
		$cmd .= " --ctimeout $optConnectTimeout" if ($optConnectTimeout);
		print $log "... call: $cmd \n" if ($pp);
		my $result = callScript($cmd);
		print $log $result if ($result and $pp);

		my $extract = 'unknown';
		$result =~ m/^([^\s]+) /;
		$extract = $1 if ($1);
		$rc = 0 if ($extract eq "OK");
		$rc = 1 if ($rc == 3 and $result =~ m/AUTHENTICATION/);
		if ($rc == 0) {
		    $exitCode = 0;
		    print "... CIM connection OK \n";
		    writeTxtFile($fileHost, "CIM", $result);
		    createCIMconfig($fileHost, "CIM", $result);
		    # remove AUTH ERROR
		    $extract = "AUTHENTICATION ERROR";
		    if ($refResults and $refResults->{"AUTHENTICATION ERROR"}) {
			my $store = $refResults->{"AUTHENTICATION ERROR"};
			if ($store =~ m/$host[|]CIM/) { # be careful with the regexp ! 
			    $store =~ s/[+ ]*$host.CIM[\s]*//;
			    $refResults->{"AUTHENTICATION ERROR"} = $store;
			}
		    }
		} # success
		elsif ($rc == 1) { # CKAE
		    $extract = "AUTHENTICATION ERROR";
		    $exitCode = 1 if ($exitCode == 3); # do not overwrite potential SNMP-OK
		    print "... CIM AUTHENTICATION ERROR \n";
		    if ($refResults and $refResults->{"AUTHENTICATION ERROR"}) {
			my $store = $refResults->{"AUTHENTICATION ERROR"};
			$refResults->{"AUTHENTICATION ERROR"} = "$store + $host|CIM"
			    if ($store !~ m/$host[|]CIM/); # be careful with the regexp ! 
		    } else {
			$refResults->{"AUTHENTICATION ERROR"} = "$host|CIM";
		    }
		}
		elsif ($result =~ m/TIMEOUT/i) {
		    $extract = "TIMEOUT";
		    if ($refResults and $refResults->{"TIMEOUT"}) {
			my $store = $refResults->{"TIMEOUT"};
			$refResults->{"TIMEOUT"} = "$store + $host|CIM";
		    } else {
			$refResults->{"TIMEOUT"} = "$host|CIM";
		    }
		}
		print $log "<<< $extract\n" if ($pp);
	    }
	    print $log "END DATE: \t" . ctime() . "\n";
	    close $log if ($log);
	} # single input file
	print "<<< $host $exitCode\n" if ($main::processPrint and $start);
  } #oneHost

  sub ipv4discovery {
	my $host = shift;
	my $infile = shift;
	my $ipv4host = $host;
	my $bestExitCode = 3;
	my $firstLimit = undef;
	my $lastLimit = undef;
	# initial check of hostaddress
	if ($ipv4host !~ m/^\d+\.\d+\.\d+\./) {
		$exitCode = 2;
		$msg .= " - This method requires IPv4 address parts <n>.<n>.<n>. as host address (be aware of the dot at the end)";
	} 
	$ipv4host =~ m/^(\d+)\.(\d+)\.(\d+)\./;
	if ($exitCode != 2 
	and ($1 > 255 or $2 > 255 or $3 > 255 or $1 < 0 or $2 < 0 or $3 < 0)) 
	{
		$exitCode = 2;
		$msg .= " - Errnous number parts in IPv4 host address";
	}
	if ($exitCode != 2 and $ipv4host =~ m/^\d+\.\d+\.\d+\.(\d+-\d+)$/) {
	    my $limits = $1;
	    $limits =~ m/(\d+)-(\d+)/;
	    $firstLimit = $1;
	    $lastLimit = $2;
	    if ($1 > 255 or $2 > 255 or $1 < 0 or $2 < 0) {
		$exitCode = 2;
		$msg .= " - Errnous number parts in IPv4 host address";
	    } elsif ($firstLimit > $lastLimit) {
		$exitCode = 2;
		$msg .= " - Errnous number limit parts in IPv4 host address";
	    }
	    $lastLimit++ if (defined $lastLimit);
	    $ipv4host =~ s/\d+-\d+$//;
	} elsif ($exitCode != 2 and $ipv4host !~ m/^\d+\.\d+\.\d+\.$/) {
		$exitCode = 2;
		$msg .= " - This method requires IPv4 part <n>.<n>.<n>.[<range>] as host address";
	}
	if ($exitCode != 2) {
		$exitCode = 3;
		my $first = 0;
		my $last = 256;
		if (defined $firstLimit and defined $lastLimit) {
		    $first = $firstLimit;
		    $last  = $lastLimit;
		}
		for (my $i=$first;$i < $last;$i++) {
			#print (">>> $ipv4host" . "$i\n");
			$optHost = "$ipv4host" . "$i";
			$exitCode = 3;
			oneHost($optHost,$infile,1);
			$bestExitCode = $exitCode if ($exitCode < $bestExitCode);
		}
		$exitCode = $bestExitCode;
	}
  } #ipv4discovery

  sub hostCollection {
	my $collectionFile = shift;
	my $infile = shift;
	my @hostline = ();

	@hostline = handleHostCollectionFile($collectionFile);
	return if ($exitCode == 2);

	foreach my $oneLine (@hostline) {
		#or  $oneLine =~ m/^\d+\.\d+\.\d+\.\d+\-\d+$/) 
		if ($oneLine =~ m/^\d+\.\d+\.\d+\.$/ 
		or  $oneLine =~ m/^\d+\.\d+\.\d+\.\d+\-\d+$/ )
		{
		    ipv4discovery($oneLine, $infile);
		} else {
		    oneHost($oneLine, $infile,1);
		}
	} # foreach
  } #hostCollection

  sub processData {
	$exitCode = 3;
        checkTools(); # set $usable* variables
	return if ($exitCode == 2);

	# the output directory
	handleOutputDirectory();
	return if ($exitCode == 2);

	print "DATE; \t" . ctime() . "\n" if ($main::processPrint);
	print "OUTDIR:\t$optOutdir\n" if ($main::processPrint);
	handleInputOptionFiles($optInputFile);
	return if ($exitCode == 2);
	
	# actions
	if ($optOneHost) {
	    oneHost($optHost, $optInputFile,1); 
	} elsif ($optIpv4Discovery) {
	    ipv4discovery($optHost, $optInputFile); 
	} elsif ($optHostCollection) {
	    hostCollection($optHostCollection, $optInputFile);
	}
	print "SUMMARY:\n" if ($main::processPrint);
	if ($refResults and $main::processPrint) {
	    my $special = undef;
	    $special = $refResults->{"TIMEOUT"};
	    if ($special) {
		print "TIMEOUT\n\t" . $special . "\n";
	    }
	    $special = $refResults->{"AUTHENTICATION ERROR"};
	    if ($special) {
		print "AUTHENTICATION ERROR\n\t" . $special . "\n";
	    }
	    $special = $refResults->{"CONNECTION OK - NO SERVERVIEW"};
	    if ($special) {
		print "CONNECTION OK - NO SERVERVIEW\n\t" . $special . "\n";
	    }
	    my %results = %$refResults;
	    foreach my $hostnm (sort %results) {
		next  if (!$hostnm);
		next if ($hostnm =~ m/AUTHENTICATION ERROR/ or $hostnm =~ m/TIMEOUT/
		    or $hostnm =~ m/CONNECTION OK \- NO SERVERVIEW/);
		my $value = $results{$hostnm};
		next  if (!$value);
		print "$hostnm\n\t" . $value . "\n" if ($hostnm);
	    }
	}
  } #processData
###############################################################################
#------------ MAIN PART
$|++; # for unbuffered stdout print (due to Perl documentation)

my $path = $0;
$main::scriptPath = $path;
$main::scriptPath =~ s/[^\/]+$//;

handleOptions();

processData();

# final output 
my $stateString = $state[$exitCode];

finalizeMessageContainer();
finalize(
	$exitCode, 
	$state[$exitCode], 
	$msg,
	(! $notifyMessage	? '': "\n" . $notifyMessage),
	(! $longMessage		? '' : "\n" . $longMessage),
	($variableVerboseMessage ? "\n" . $variableVerboseMessage : ""),
);
################ EOSCRIPT
